"""
Template Manager Widgets
========================

Specialized widgets for template management operations.
"""

from .template_creator import TemplateCreatorWidget
from .template_manager import TemplateManagerWidget

__all__ = [
    'TemplateCreatorWidget', 
    'TemplateManagerWidget'
]