"""
Analysis Features Package

This package contains all analysis-related features for the SNID GUI.
"""

from .games_integration import GamesIntegration
from .analysis_controller import AnalysisController

__all__ = ['GamesIntegration', 'AnalysisController'] 
