"""
SNID SAGE GUI Package
=====================

Graphical user interfaces for SNID SAGE (SuperNova IDentification – Spectral Analysis and Guided Exploration).

Version 1.0.0 - Developed by Fiorenzo Stoppa
Based on the original Fortran SNID by Stéphane Blondin & John L. Tonry
"""

__version__ = "1.0.0"
__author__ = "Fiorenzo Stoppa" 
